﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;
using System.Diagnostics;

// @Name: ao_dc_voltage.cs
// @Author: Smacq
// @Copyright: Smacq
// @Date: 2025/02/08
// @Brief:
// This code focuses on the Ao functions of the USB - 5000 device, outputting fixed voltages.
// It sets the device's A0 to 1V, A1 to 2V, A2 to 3V, and A3 to 4V.
// The analog voltage range is -10V ~ +10V.
// The Ao channels vary depending on the module, either Ao_4 or NONE.
// You can refer to the user manual for more information about functions or device details.
//
// Execution result when only connecting USB for USB - 5100:
// 0
// 0 AOChannel_0 is 1V
// 0 AOChannel_1 is 2V
// 0 AOChannel_2 is 3V
// 0 AOChannel_3 is 4V
// 0

class Program
{
    static void Main(string[] args)
    {
        // Device index starts from zero
        int DevIndex = 0;

        // Ao channel, Ao channel 0 to Ao channel 3
        byte AOChannel_0 = 0;
        byte AOChannel_1 = 1;
        byte AOChannel_2 = 2;
        byte AOChannel_3 = 3;

        // voltage value, range is -10 to +10
        float voltage_4_5 = 4.5f;
        float voltage_4 = 4.0f;
        float voltage_3 = 3.0f;
        float voltage_2 = 2.0f;
        float voltage_1 = 1.0f;
        float voltage_0 = 0.0f;

        int error_code;

        // Open specific device and print return value.
        error_code = USB5000DLL.USB5OpenDevice(DevIndex);
        Console.WriteLine(error_code);

        // Set single Ao channel
        error_code = USB5000DLL.SetUSB5AoImmediately(DevIndex, AOChannel_0, voltage_1);
        Console.WriteLine(error_code + " AOChannel_0 is 1V");
        error_code = USB5000DLL.SetUSB5AoImmediately(DevIndex, AOChannel_1, voltage_2);
        Console.WriteLine(error_code + " AOChannel_1 is 2V");
        error_code = USB5000DLL.SetUSB5AoImmediately(DevIndex, AOChannel_2, voltage_3);
        Console.WriteLine(error_code + " AOChannel_2 is 3V");
        error_code = USB5000DLL.SetUSB5AoImmediately(DevIndex, AOChannel_3, voltage_4);
        Console.WriteLine(error_code + " AOChannel_3 is 4V");

        Console.WriteLine("Press Enter to continue...");
        Console.ReadLine();
        // Close Device
        error_code = USB5000DLL.USB5CloseDevice(DevIndex);
        Console.WriteLine(error_code);
    }

    class USB5000DLL
    {
        [DllImport("USB5000.dll")]
        public static extern int FindUSB5DAQ();
        [DllImport("USB5000.dll")]
        public static extern int USB5OpenDevice(int DevIndex);
        [DllImport("USB5000.dll")]
        public static extern int USB5CloseDevice(int DevIndex);
        [DllImport("USB5000.dll")]
        public static extern int USB5ReadDevcieSN(int DevIndex);
        [DllImport("USB5000.dll")]
        public static extern int USB5ReadDevcieModel(int DevIndex);

        //--------------------------------------------------------------------------
        // Ananlog Input Configuration

        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AiSampleRate(int DevIndex, uint SamplePeriod);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AiSampleMode(int DevIndex, byte AiSampleMode);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AiConnectType(int DevIndex, byte AiConnectType);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AiRange(int DevIndex, byte Chan, float AiRange);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AiChanSel(int DevIndex, byte Chan, byte Sel);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AiTrigSource(int DevIndex, byte AiTrigSource);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AiConvSource(int DevIndex, byte AiConvSource);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AiPreTrigPoints(int DevIndex, uint AiPreTrigPoints);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AiOneShotPoints(int DevIndex, uint AiOneShotPoints);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5ClrAiFifo(int DevIndex);

        //--------------------------------------------------------------------------
        // Digital I/O Configuration

        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DiSampleRate(int DevIndex, uint SamplePeriod);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DiSampleMode(int DevIndex, byte DiSampleMode);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DiTrigSource(int DevIndex, byte DiTrigSource);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DiConvSource(int DevIndex, byte DiConvSource);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DiPreTrigPoints(int DevIndex, uint DiPreTrigPoints);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DiOneShotPoints(int DevIndex, uint DiOneShotPoints);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5ClrDiFifo(int DevIndex);

        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DoSampleRate(int DevIndex, uint SamplePeriod);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DoSampleMode(int DevIndex, byte DoSampleMode);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DoTrigSource(int DevIndex, byte DoTrigSource);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DoConvSource(int DevIndex, byte DoConvSource);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DoCycle(int DevIndex, uint DoCycle);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DoDataFifo(int DevIndex, uint[] Value, uint Len);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5ClrDoFifo(int DevIndex);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DoWaveCtrl(int DevIndex, uint Chan);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DoImmediately(int DevIndex, uint Chan, uint Value);

        //--------------------------------------------------------------------------
        // Ananlog Output Configuration

        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AoSampleRate(int DevIndex, byte Chan, uint SamplePeriod);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AoSampleMode(int DevIndex, byte Chan, byte AoSampleMode);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AoTrigSource(int DevIndex, byte Chan, byte AoTrigSource);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AoConvSource(int DevIndex, byte Chan, byte AoConvSource);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AoCycle(int DevIndex, byte Chan, byte AoCycle);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AoDataFifo(int DevIndex, byte Chan, float[] Voltage, uint Len);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5ClrAoFifo(int DevIndex, byte Chan);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AoSync(int DevIndex, byte Chans);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AoImmediately(int DevIndex, byte Chan, float Voltage);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AoImmediately4Chans(int DevIndex, float Volt0, float Volt1, float Volt2, float Volt3);

        //--------------------------------------------------------------------------
        // Trig Control

        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AiSoftTrig(int DevIndex);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DiSoftTrig(int DevIndex);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5DoSoftTrig(int DevIndex);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5AoSoftTrig(int DevIndex, byte Chan);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5GlobalSoftTrig(int DevIndex);

        [DllImport("USB5000.dll")]
        public static extern int SetUSB5ClrTrigger(int DevIndex);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5ClrAiTrigger(int DevIndex);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5ClrDiTrigger(int DevIndex);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5ClrDoTrigger(int DevIndex);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5ClrAoTrigger(int DevIndex, byte Chan);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5ClrGlobalSoftTrig(int DevIndex);

        //--------------------------------------------------------------------------
        // Sync Configuration

        [DllImport("USB5000.dll")]
        public static extern int SetUSB5ExtTrigOutSource(int DevIndex, byte Source);
        [DllImport("USB5000.dll")]
        public static extern int SetUSB5ExtConvOutSource(int DevIndex, byte Source);

        //--------------------------------------------------------------------------
        // Get Data Acquired

        [DllImport("USB5000.dll")]
        public static extern int USB5GetAi(int DevIndex, uint Points, float[] Ai, int TimeOut);
        [DllImport("USB5000.dll")]
        public static extern int USB5GetDi(int DevIndex, uint Points, byte[] Di, int TimeOut);
    }
}